<?php

/**
 * @file
 * views_aggregator_functions.inc
 *
 * The complete set of views aggregation functions that come with the module.
 * Note that other modules can add their own functions by implementing
 * hook_views_aggregation_functions_info()
 */

/**
 * Implements hook_views_aggregation_functions_info().
 *
 * @return array
 *   array of available aggregation function names with descriptions to display
 *   in the Views UI
 */
function views_aggregator_views_aggregation_functions_info() {
  $functions = array(
    // The first two are special and must be executed before the others.
    'views_aggregator_row_filter' => array(
      'group' => t('Filter rows (by regexp)'),
      'column' => NULL,
    ),
    'views_aggregator_group_and_compress' => array(
      'group' => t('Group and compress'),
      'column' => NULL,
    ),

    /* Regular aggregation functions star here */

    'views_aggregator_average' => array(
      'group' => t('Average'),
      'column' => t('Average'),
    ),
    'views_aggregator_count' => array(
      'group' => t('Count (having regexp)'),
      'column' => t('Count (having regexp)')
    ),
    'views_aggregator_first' => array(
      'group' => t('Display first member'),
      'column' => NULL,
    ),
    'views_aggregator_enumerate' => array(
      'group' => t('Enumerate'),
      'column' => NULL,
    ),
    'views_aggregator_replace' => array(
      'group' => t('Label (enter below)'),
      'column' => t('Label (enter below)'),
    ),
    'views_aggregator_maximum' => array(
      'group' => t('Maximum'),
      'column' => t('Maximum'),
    ),
    'views_aggregator_minimum' => array(
      'group' => t('Minimum'),
      'column' => t('Minimum'),
    ),
    'views_aggregator_sum' => array(
      'group' => t('Sum'),
      'column' => t('Sum'),
    ),
    'views_aggregator_tally' => array(
      'group' => t('Tally members'),
      'column' => NULL,
    ),
  );
  return $functions;
}

/**
 * Keeps only the groups that match the regular expression filter.
 *
 * Matching takes place on the raw values (1000, rather than "$ 1,000").
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to count groups members in
 * @parm string $regexp
 *   if empty all result rows are kept
 *
 * @return array
 *   a subset of the original array of groups
 */
function views_aggregator_row_filter($views_plugin_style, $field_handler, $regexp = NULL) {
  if (empty($regexp)) {
    return;
  }
  if (preg_match('/[a-zA-Z0-9_]+/', $regexp)) {
    // Interprt omitted brace chars in the regexp as a verbatim text match.
    $regexp = "/$regexp/";
  }
  foreach ($views_plugin_style->view->result as $num => $row) {
    $field_value = views_aggregator_get_cell($field_handler, $num, FALSE);
    if (!preg_match($regexp, $field_value)) {
      //$field_value = views_aggregator_get_cell($field_handler, $num, TRUE);
      //if (!preg_match($regexp, $field_value)) {
      unset($views_plugin_style->rendered_fields[$num]);
      unset($views_plugin_style->view->result[$num]);
      //}
    }
  }
}

/**
 * Aggregates the supplied view results into grouped rows.
 *
 * This function must be selected for one column (field) in the results table.
 * Its parameters and return value are different from the other aggregation
 * functions.
 *
 * @param object $view_results
 *   the result rows as they appear on the view object
 * @param object $field_handler
 *   the handler for the view column to group rows on
 *
 * @return array
 *   an array of groups, keyed by group value first and row number second.
 */
function views_aggregator_group_and_compress($view_results, $field_handler) {
  $groups = array();
  foreach ($view_results as $num => $row) {
    $group_value = views_aggregator_get_cell($field_handler, $num, TRUE);
    $groups[$group_value][$num] = $row;
  }
  return $groups;
}

/**
 * Aggregates a group of result rows as the minimum amongst its members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find the minimum in
 *
 * @return array
 *   an array of column values, one for each group and one for the column.
 */
function views_aggregator_average($groups, $field_handler) {
  $values = array();
  $sum_column = 0.0;
  $count_all_rows = 0;
  foreach ($groups as $group => $rows) {
    $sum = 0.0;
    foreach ($rows as $num => $row) {
      $sum += views_aggregator_get_cell($field_handler, $num, FALSE);
    }
    $values[$group] = $sum / count($rows);
    $sum_column += $sum;
    $count_all_rows += count($rows);
  }
  $values['column'] = $sum_column / $count_all_rows;
  return $values;
}

/**
 * Aggregates a group of result rows as a count of the number of group members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to count groups members in
 * @parm string $value
 *   an optional value to count, if omitted all group values count
 *
 * @return array
 *   an array of column values, one for each group and one for the column
 */
function views_aggregator_count($groups, $field_handler, $group_regexp = NULL, $column_regexp = NULL) {
  $values = array();
  $count_column = 0;
  $regexp = isset($group_regexp) ? $group_regexp : $column_regexp;
  if (preg_match('/[a-zA-Z0-9_]+/', $regexp)) {
    // Interprt omitted brace chars in the regexp as a verbatim text match.
    $regexp = "/$regexp/";
  }
  foreach ($groups as $group => $rows) {
    if (empty($regexp)) {
      $count = count($rows);
    }
    else {
      $count = 0;
      foreach ($rows as $num => $row) {
        $cell = views_aggregator_get_cell($field_handler, $num, TRUE);
        if (preg_match($regexp, $cell)) {
          $count++;
        }
      }
    }
    $values[$group] = $count;
    $count_column += $count;
  }
  $values['column'] = $count_column;
  return $values;
}

/**
 * Aggregates a group of result rows as the enumeration of its members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find members of the group
 *
 * @return array
 *   an array of column values, one for each group.
 */
function views_aggregator_enumerate($groups, $field_handler) {
  $values = array();
  foreach ($groups as $group => $rows) {
    $cell_values = array();
    foreach ($rows as $num => $row) {
      $cell = views_aggregator_get_cell($field_handler, $num, TRUE);
      if (!empty($cell) && !in_array($cell, $cell_values)) {
        $cell_values[] = $cell;
      }
    }
    if (count($cell_values) > 1) {
      // After grouping the fields in the group no longer belong to one
      // entity. Cannot easily support hyper-linking, so switch it off.
      unset($field_handler->options['link_to_node']);
      @sort($cell_values, SORT_NATURAL | SORT_FLAG_CASE);
    }
    $values[$group] = implode('<br/>', $cell_values);
  }
  return $values;
}

/**
 * Aggregates a group of result rows as the first member of the group.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find the first group member in
 *
 * @return array
 *   an empty array
 */
function views_aggregator_first($groups, $field_handler) {
  // This is the default operation anyway, so nothing to do.
  return array();
}

/**
 * Aggregates a group of result rows as the maximum across its members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find the maximum groups member in
 *
 * @return array
 *   an array of column values, one for each group, plus the 'column' group
 */
function views_aggregator_maximum($groups, $field_handler) {
  $values = array();
  foreach ($groups as $group => $rows) {
    $is_first = TRUE;
    foreach ($rows as $num => $row) {
      $value = views_aggregator_get_cell($field_handler, $num, FALSE);
      if ($is_first) {
        $value = $maximum;
        $is_first = FALSE;
      }
      elseif ($value > $maximum) {
        $maximum = $value;
      }
    };
    $values[$group] = $maximum;
    if (!isset($maximum_column) || $maximum > $maximum_column) {
      $maximum_column = $maximum;
    }
  }
  $values['column'] = $maximum_column;
  return $values;
}

/**
 * Aggregates a group of result rows as the minimum across its members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find the minimum groups member in
 *
 * @return array
 *   an array of column values, one for each group, plus the 'column' group
 */
function views_aggregator_minimum($groups, $field_handler) {
  $values = array();
  foreach ($groups as $group => $rows) {
    $is_first = TRUE;
    foreach ($rows as $num => $row) {
      $value = views_aggregator_get_cell($field_handler, $num, FALSE);
      if ($is_first) {
        $value = $minimum;
        $is_first = FALSE;
      }
      elseif ($value < $minimum) {
        $minimum = $value;
      }
    };
    $values[$group] = $minimum;
    if (!isset($minimum_column) || $minimum < $minimum_column) {
      $minimum_column = $minimum;
    }
  }
  $values['column'] = $minimum_column;
  return $values;
}

/**
 * Aggregates a group of result rows as a word, phrase or number.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column
 * @param string $replace_text
 *   an optional parameter, specifying the replacement text to use
 * @param string $column_label
 *   an optional parameter, specifying a label placed the bottom of the column
 *
 * @return array
 *   an array of column values, one for each group and one for the 'column'
 */
function views_aggregator_replace($groups, $field_handler, $replace_text = NULL, $column_label = NULL) {
  $values = array();
  foreach ($groups as $group => $rows) {
    if (isset($replace_text)) {
      $values[$group] = $replace_text;
    }
    if (count($rows) > 1 && isset($replace_text)) {
      // With more than one field in a group the fields no longer belong to one
      // particular entity. Cannot support hyper-linking, so switch it off.
      // Unfortunately this applies to the entire column.
      unset($field_handler->options['link_to_node']);
    }
  }
  if (isset($column_label)) {
    $values['column'] = $column_label;
  }
  return $values;
}

/**
 * Aggregates a group of result rows as a sum.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to sum groups in
 *
 * @return array
 *   an array of column values, one for each group, plus the 'column' group
 */
function views_aggregator_sum($groups, $field_handler) {
  $values = array();
  $sum_column = 0.0;
  foreach ($groups as $group => $rows) {
    $sum = 0.0;
    foreach ($rows as $num => $row) {
      $sum += views_aggregator_get_cell($field_handler, $num, FALSE);
    };
    $values[$group] = $sum;
    $sum_column += $sum;
  }
  $values['column'] = $sum_column;
  return $values;
}

/**
 * Aggregates a group of result rows as the tally of its members.
 *
 * @param array groups
 *   an array of groups of rows, each group indexed by group value
 * @param object $field_handler
 *   the handler for the view column to find members of the group
 *
 * @return array
 *   an array of column values, one for each group
 */
function views_aggregator_tally($groups, $field_handler) {
  $values = array();
  foreach ($groups as $group => $rows) {
    $tally = array();
    foreach ($rows as $num => $row) {
      $cell = views_aggregator_get_cell($field_handler, $num, TRUE);
      if (empty($cell)) {
        // Not tallying empty values.
        $values[$group] = NULL;
        break;
      }
      if (isset($tally[$cell])) {
        $tally[$cell]++;
      }
      else {
        $tally[$cell] = 1;
      }
    }
    if (count($tally) > 1) {
      // With more than one field in a group the fields no longer belong to one
      // particular entity. Cannot support hyper-linking, so switch it off.
      // Unfortunately this applies to the entire column.
      unset($field_handler->options['link_to_node']);
    }
    @ksort($tally, SORT_NATURAL | SORT_FLAG_CASE);
    $rendered_tally = array();
    foreach ($tally as $cell => $count) {
      $rendered_tally[] = "$cell ($count)";
    }
    $values[$group] = implode('<br/>', $rendered_tally);
  }
  return $values;
}